'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _KeyCode = require('rc-util/lib/KeyCode');

var _KeyCode2 = _interopRequireDefault(_KeyCode);

var _utils = require('./utils');

var _WheelHandler = require('./dom/WheelHandler');

var _WheelHandler2 = _interopRequireDefault(_WheelHandler);

var _MouseMoveTracker = require('./dom/MouseMoveTracker');

var _MouseMoveTracker2 = _interopRequireDefault(_MouseMoveTracker);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var FACE_MARGIN = 4; /* eslint-disable react/require-default-props */

var FACE_MARGIN_2X = FACE_MARGIN * 2;
var SCROLLBAR_SIZE = 15;
var FACE_SIZE_MIN = 30;
var KEYBOARD_SCROLL_AMOUNT = 40;

var lastScrolledScrollbar = null;

var Scrollbar = function (_React$Component) {
  (0, _inherits3.default)(Scrollbar, _React$Component);

  function Scrollbar(props) {
    (0, _classCallCheck3.default)(this, Scrollbar);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Scrollbar.__proto__ || (0, _getPrototypeOf2.default)(Scrollbar)).call(this, props));

    _this.onWheel = function (delta) {
      _this.setNextState(_this.calculateState(_this.state.position + delta));
    };

    _this.onWheelX = function (deltaX) {
      _this.onWheel(deltaX);
    };

    _this.onWheelY = function (deltaX, deltaY) {
      _this.onWheel(deltaY);
    };

    _this.onMouseDown = function (e) {
      var nextState = void 0;

      if (e.target !== _this.faceElem) {
        var nativeEvent = e.nativeEvent;
        var position = _this.state.horizontal ? nativeEvent.offsetX || nativeEvent.layerX : nativeEvent.offsetY || nativeEvent.layerY;

        // mousedown on the scroll-track directly, move the
        // center of the scroll-face to the mouse position.
        position /= _this.state.scale;
        nextState = _this.calculateState(position - _this.state.faceSize * 0.5 / _this.state.scale);
      } else {
        nextState = {};
      }

      nextState.focused = true;

      _this.setNextState(nextState);
      _this.mouseMoveTracker.capture(e);
      // focus the container so it may receive keyboard events
      _this.containerElem.focus();
    };

    _this.onMouseMove = function (deltaX, deltaY) {
      var delta = _this.state.horizontal ? deltaX : deltaY;
      if (delta !== 0) {
        delta /= _this.state.scale;
        _this.setNextState(_this.calculateState(_this.state.position + delta));
      }
    };

    _this.onMouseMoveEnd = function () {
      _this.mouseMoveTracker.release();
      _this.setState({ dragging: false, focused: false });
    };

    _this.onKeyDown = function (e) {
      var keyCode = e.keyCode;

      // let focus move off the scrollbar
      if (keyCode === _KeyCode2.default.TAB) {
        return;
      }

      var distance = KEYBOARD_SCROLL_AMOUNT;
      var direction = 0;

      if (_this.state.horizontal) {
        switch (keyCode) {
          case _KeyCode2.default.HOME:
            direction = -1;
            distance = _this.props.contentSize;
            break;

          case _KeyCode2.default.LEFT:
            direction = -1;
            break;

          case _KeyCode2.default.RIGHT:
            direction = 1;
            break;

          default:
            return;
        }
      } else {
        switch (keyCode) {
          case _KeyCode2.default.SPACE:
            if (e.shiftKey) {
              direction = -1;
            } else {
              direction = 1;
            }
            break;

          case _KeyCode2.default.HOME:
            direction = -1;
            distance = _this.props.contentSize;
            break;

          case _KeyCode2.default.UP:
            direction = -1;
            break;

          case _KeyCode2.default.DOWN:
            direction = 1;
            break;

          case _KeyCode2.default.PAGE_UP:
            direction = -1;
            distance = _this.props.size;
            break;

          case _KeyCode2.default.PAGE_DOWN:
            direction = 1;
            distance = _this.props.size;
            break;

          default:
            return;
        }
      }

      e.preventDefault();

      _this.setNextState(_this.calculateState(_this.state.position + distance * direction));
    };

    _this.onFocus = function () {
      _this.setState({ focused: true });
    };

    _this.onBlur = function () {
      _this.setState({ focused: false });
    };

    _this.shouldHandleScrollX = function (delta) {
      return _this.props.orientation === 'horizontal' ? _this.shouldHandleScrollChange(delta) : false;
    };

    _this.shouldHandleScrollY = function (delta) {
      return _this.props.orientation !== 'horizontal' ? _this.shouldHandleScrollChange(delta) : false;
    };

    _this.triggerOnScrollCallback = function () {
      var position = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _this.state.position;

      if (_this.props.onScroll) {
        _this.props.onScroll(position);
      }
    };

    _this.state = _this.calculateState(props.position || props.defaultPosition || 0);
    return _this;
  }

  (0, _createClass3.default)(Scrollbar, [{
    key: 'componentWillMount',
    value: function componentWillMount() {
      var onWheel = this.state.horizontal ? this.onWheelX : this.onWheelY;

      this.wheelHandler = new _WheelHandler2.default(onWheel, this.shouldHandleScrollX, this.shouldHandleScrollY);

      this.initialRender = true;
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.mouseMoveTracker = new _MouseMoveTracker2.default(document.documentElement, this.onMouseMove, this.onMouseMoveEnd);

      if (this.props.position !== undefined && this.state.position !== this.props.position) {
        this.triggerOnScrollCallback();
      }

      this.initialRender = false;
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      if (!(0, _utils.shallowEqual)(this.props, nextProps)) {
        var controlledPosition = nextProps.position;
        if (controlledPosition === undefined) {
          this.setNextState(this.calculateState(this.state.position, nextProps));
        } else {
          this.setNextState(this.calculateState(controlledPosition, nextProps), nextProps);
        }
      }
    }
  }, {
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps, nextState) {
      return !(0, _utils.shallowEqual)(this.props, nextProps) || !(0, _utils.shallowEqual)(this.state, nextState);
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this.mouseMoveTracker.release();
      delete this.mouseMoveTracker;

      if (lastScrolledScrollbar === this) {
        lastScrolledScrollbar = null;
      }
    }
  }, {
    key: 'setNextState',
    value: function setNextState(nextState) {
      var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.props;

      var controlledPosition = props.position;
      var willScroll = this.state.position !== nextState.position;
      var callback = willScroll ? this.triggerOnScrollCallback : undefined;

      if (controlledPosition === undefined) {
        this.setState(nextState, callback);
      } else if (controlledPosition === nextState.position) {
        this.setState(nextState);
      } else {
        // Scrolling is controlled.
        // Don't update the state and let the owner to update the scrollbar instead.
        if (nextState.position !== undefined && nextState.position !== this.state.position) {
          callback(nextState.position);
        }
        return;
      }

      if (willScroll && lastScrolledScrollbar !== this) {
        if (lastScrolledScrollbar) {
          lastScrolledScrollbar.blur();
        }
        lastScrolledScrollbar = this;
      }
    }
  }, {
    key: 'calculateState',
    value: function calculateState(position) {
      var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.props;
      var size = props.size,
          contentSize = props.contentSize,
          orientation = props.orientation;

      // unscrollable

      if (size < 1 || contentSize <= size) {
        return {
          position: 0,
          scrollable: false
        };
      }

      var cachedStateKey = position + '_' + size + '_' + contentSize + '_' + orientation;
      if (this.cachedStateKey === cachedStateKey) {
        return this.cachedState;
      }

      // There are two types of positions here.
      // 1) Phisical position: changed by mouse or keyboard
      // 2) Logical position: changed by props.
      // The logical position will be kept as as internal state and the `render()`
      // function will translate it into physical position to render.

      var scale = size / contentSize;
      var faceSize = size * scale;

      if (faceSize < FACE_SIZE_MIN) {
        scale = (size - FACE_SIZE_MIN) / (contentSize - size);
        faceSize = FACE_SIZE_MIN;
      }

      var maxPosition = contentSize - size;

      position = (0, _utils.clamp)(position, 0, maxPosition); // eslint-disable-line

      // This function should only return flat values that can be
      // compared quiclky by `shallowEqual`.
      var state = {
        scale: scale,
        faceSize: faceSize,
        position: position,
        dragging: this.mouseMoveTracker ? this.mouseMoveTracker.isDragging() : false,
        horizontal: orientation === 'horizontal',
        scrollable: true

        // cache the state
      };this.cachedStateKey = cachedStateKey;
      this.cachedState = state;

      return state;
    }
  }, {
    key: 'shouldHandleScrollChange',
    value: function shouldHandleScrollChange(delta) {
      var nextState = this.calculateState(this.state.position + delta);
      return nextState.position !== this.state.position;
    }
  }, {
    key: 'blur',
    value: function blur() {
      if (!this.containerElem) {
        return;
      }

      try {
        this.onBlur();
        this.containerElem.blur();
      } catch (oops) {
        // pass
      }
    }
  }, {
    key: 'scrollBy',
    value: function scrollBy(delta) {
      this.onWheel(delta);
    }
  }, {
    key: 'render',
    value: function render() {
      var _classNames,
          _this2 = this;

      if (!this.state.scrollable) {
        return null;
      }

      var _props = this.props,
          prefixCls = _props.prefixCls,
          size = _props.size,
          opaque = _props.opaque;

      var faceSize = this.state.faceSize;
      var position = this.state.position * this.state.scale + FACE_MARGIN;
      var active = this.state.focused || this.state.dragging;
      var horizontal = this.state.horizontal;

      var mainClassName = (0, _classnames2.default)((_classNames = {}, (0, _defineProperty3.default)(_classNames, prefixCls + '-scroll', true), (0, _defineProperty3.default)(_classNames, 'vertical', !horizontal), (0, _defineProperty3.default)(_classNames, 'horizontal', horizontal), (0, _defineProperty3.default)(_classNames, 'opaque', opaque), (0, _defineProperty3.default)(_classNames, 'active', active), _classNames));

      var faceClassName = (0, _classnames2.default)((0, _defineProperty3.default)({}, prefixCls + '-scroll-face', true));

      var mainStyle = void 0;
      var faceStyle = void 0;

      if (horizontal) {
        mainStyle = {
          left: this.props.left,
          width: size,
          height: SCROLLBAR_SIZE
        };
        faceStyle = {
          width: faceSize - FACE_MARGIN_2X
        };
        (0, _utils.translateDOMPosition)(faceStyle, position, 0, this.initialRender);
      } else {
        mainStyle = {
          top: this.props.top,
          width: SCROLLBAR_SIZE,
          height: size
        };
        faceStyle = {
          height: faceSize - FACE_MARGIN_2X
        };
        (0, _utils.translateDOMPosition)(faceStyle, 0, position, this.initialRender);
      }

      if (this.props.zIndex) {
        mainStyle.zIndex = this.props.zIndex;
      }

      if (this.props.trackColor === 'gray') {
        mainStyle.backgroundColor = '#f6f7f8';
      }

      return _react2.default.createElement(
        'div',
        { // eslint-disable-line
          style: mainStyle,
          className: mainClassName,
          tabIndex: 0 // eslint-disable-line
          , onBlur: this.onBlur,
          onFocus: this.onFocus,
          onKeyDown: this.onKeyDown,
          onMouseDown: this.onMouseDown,
          onWheel: this.wheelHandler.onWheel,
          ref: function ref(containerElem) {
            _this2.containerElem = containerElem;
          }
        },
        _react2.default.createElement('div', {
          style: faceStyle,
          className: faceClassName,
          ref: function ref(faceElem) {
            _this2.faceElem = faceElem;
          }
        })
      );
    }
  }]);
  return Scrollbar;
}(_react2.default.Component);

Scrollbar.SIZE = SCROLLBAR_SIZE;
Scrollbar.OFFSET = FACE_MARGIN / 2 + 1;
Scrollbar.KEYBOARD_SCROLL_AMOUNT = KEYBOARD_SCROLL_AMOUNT;
Scrollbar.propTypes = {
  size: _propTypes2.default.number.isRequired,
  contentSize: _propTypes2.default.number.isRequired,
  position: _propTypes2.default.number,
  defaultPosition: _propTypes2.default.number,
  opaque: _propTypes2.default.bool,
  orientation: _propTypes2.default.oneOf(['vertical', 'horizontal']),
  trackColor: _propTypes2.default.oneOf(['gray']),
  zIndex: _propTypes2.default.number,
  top: _propTypes2.default.number, // top position of vertical bar
  left: _propTypes2.default.number, // left position of horizontal bar
  onScroll: _propTypes2.default.func
};
Scrollbar.defaultProps = {
  defaultPosition: 0,
  opaque: true,
  zIndex: 99,
  orientation: 'vertical'
};
exports.default = Scrollbar;
module.exports = exports['default'];