"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require("babel-runtime/helpers/createClass");

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultComparator = function defaultComparator(a, b) {
  return a < b;
};

var Heap = function () {
  function Heap(items, comparator) {
    (0, _classCallCheck3.default)(this, Heap);

    this.items = items || [];
    this.size = this.items.length;
    this.comparator = comparator || defaultComparator;

    // heapify
    for (var i = Math.floor((this.size + 1) / 2); i >= 0; i -= 1) {
      this.sinkDown(i);
    }
  }

  (0, _createClass3.default)(Heap, [{
    key: "isEmpty",
    value: function isEmpty() {
      return this.size === 0;
    }
  }, {
    key: "getSize",
    value: function getSize() {
      return this.size;
    }
  }, {
    key: "peek",
    value: function peek() {
      return this.size > 0 ? this.items[0] : null;
    }
  }, {
    key: "pop",
    value: function pop() {
      if (this.size === 0) {
        return null;
      }

      var currentItem = this.items[0];
      var lastItem = this.items.pop();

      this.size -= 1;

      if (this.size > 0) {
        this.items[0] = lastItem;
        this.sinkDown(0);
      }

      return currentItem;
    }
  }, {
    key: "push",
    value: function push(item) {
      this.items[this.size] = item;
      this.size += 1;
      this.bubbleUp(this.size - 1);
    }
  }, {
    key: "bubbleUp",
    value: function bubbleUp(index) {
      var currentItem = this.items[index];
      while (index > 0) {
        var parentIndex = Math.floor((index + 1) / 2) - 1;
        var parentItem = this.items[parentIndex];

        // if parentItem < currentItem, stop
        if (this.comparator(parentItem, currentItem)) {
          return;
        }

        // swap
        this.items[parentIndex] = currentItem;
        this.items[index] = parentItem;
        index = parentIndex; // eslint-disable-line
      }
    }
  }, {
    key: "sinkDown",
    value: function sinkDown(index) {
      var currentItem = this.items[index];

      while (true) {
        // eslint-disable-line
        var leftChildIndex = 2 * (index + 1) - 1;
        var rightChildIndex = 2 * (index + 1);
        var swapIndex = -1;

        if (leftChildIndex < this.size) {
          var leftChild = this.items[leftChildIndex];
          if (this.comparator(leftChild, currentItem)) {
            swapIndex = leftChildIndex;
          }
        }

        if (rightChildIndex < this.size) {
          var rightChild = this.items[rightChildIndex];
          if (this.comparator(rightChild, currentItem)) {
            if (swapIndex === -1 || this.comparator(rightChild, this.items[swapIndex])) {
              swapIndex = rightChildIndex;
            }
          }
        }

        // if we don't have a swap, stop
        if (swapIndex === -1) {
          return;
        }

        this.items[index] = this.items[swapIndex];
        this.items[swapIndex] = currentItem;
        index = swapIndex; // eslint-disable-line
      }
    }
  }]);
  return Heap;
}();

exports.default = Heap;
module.exports = exports["default"];